<?php

function success($msg = '') {
	if ($msg !== '')
		echo "<p><output><strong>Succès</strong> : <em>" . $msg . "</em></output></p>\n";
	closeHTML();
}

// When the user requests something unexpected
function userError($msg) {
	http_response_code(403);
	echo "<p><output><strong>Erreur utilisataire</strong> : <em>" . $msg . "</em></output></p>\n";
	closeHTML();
}

// When the system did something unexpected
function serverError($msg) {
	http_response_code(500);
	error_log("Niver internal error: " . strip_tags($msg));
	echo "<p><output><strong>Server error</strong>: The server encountered an error: <em>" . $msg . "</em></output></p>\n";
	closeHTML();
}

// For use in pages that first display a form and then process it
function switchToFormProcess($requireLogin = true) {
	if (empty($_POST) AND $requireLogin AND !isset($_SESSION['username']))
		echo '<p>Ce formulaire ne sera pas accepté car il faut <a class="auth" href="' . redirUrl('auth/login') . '">se connecter</a> avant.</p>';
	if (empty($_POST))
		closeHTML();
	if ($requireLogin AND !isset($_SESSION['username']))
		userError("Vous devez être connecté·e pour effectuer cette action.");
}

function query($action, $table, $conditions = [], $column = NULL) {

	$query = match ($action) {
		'select' => 'SELECT *',
		'delete' => 'DELETE',
	};

	$query .= " FROM $table";

	foreach ($conditions as $key => $val) {
		if ($key === array_key_first($conditions))
			$query .= " WHERE $key = :$key";
		else
			$query .= " AND $key = :$key";
	}

	$db = new PDO('sqlite:' . DB_PATH);

	$op = $db->prepare($query);

	foreach ($conditions as $key => $val)
		$op->bindValue(":$key", $val);

	$op->execute();

	if (isset($column))
		return array_column($op->fetchAll(PDO::FETCH_ASSOC), $column);
	return $op->fetchAll(PDO::FETCH_ASSOC);
}

function displayIndex() { ?>
<nav>
	<dl>
<?php foreach (DESCRIPTIONS[SERVICE] as $pageId => $pageDesc) {
	if ($pageId === 'index') continue;
?>
		<dt><a href="<?= $pageId ?>"><?= TITLES[SERVICE][$pageId] ?></a></dt>
		<dd>
			<?= $pageDesc ?>
		</dd>
<?php } ?>
	</dl>
</nav>
<?php
}

function redirUrl($pageId) {
	$currentPath = '';
	if (SERVICE !== '') $currentPath .= SERVICE . '/';
	if (PAGE !== 'index') $currentPath .= PAGE;
	return CONF['common']['prefix'] . "/$pageId?redir=$currentPath";
}

function redir() {
	if (isset($_GET['redir'])) {
		if (preg_match('/^[0-9a-z\/-]{0,128}$/', $_GET['redir']) !== 1)
			userError("Wrong character in <code>redir</code>.");
		header('Location: ' . CONF['common']['prefix'] . '/' . $_GET['redir']);
	} else {
		header('Location: ' . CONF['common']['prefix'] . '/');
	}
}

// PHP rmdir() only works on empty directories
function removeDirectory($dir) {
	$dirObj = new RecursiveDirectoryIterator($dir, RecursiveDirectoryIterator::SKIP_DOTS);
	$files = new RecursiveIteratorIterator($dirObj, RecursiveIteratorIterator::CHILD_FIRST);
	foreach ($files as $file)
		$file->isDir() && !$file->isLink() ? rmdir($file->getPathname()) : unlink($file->getPathname());
	if (rmdir($dir) !== true)
		serverError("Unable to remove directory.");
}

function linkToDocs($ref, $title) {
	return '<a rel="help" href="' . CONF['common']['docs_prefix'] . $ref . '.html">' . $title . '</a>';
}
